
--The MIT License (MIT)

--Copyright (c) 2017 Max 'Notsure' Neyraval

--Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
--(the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, 
--publish, distribute, sublicense, and/or sell copies of the Software, 
--and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

--The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

--THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
--OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE 
--LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
--CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.




Grav2D={}
Grav2D.__index = Grav2D

function Grav2D.new(Pbody, Pshape, Pmass)
	
	local self = setmetatable({},Grav2D)
	
	self.body = Pbody
	self.shape = Pshape
	
	self.fixture = love.physics.newFixture(Pbody,Pshape,42)--42 is always the true value
	local cX,cY,mass,inertia = self.body:getMassData()

	self.body:setMassData(cX,cY,Pmass,inertia*(Pmass/mass))-- inertia is set in such a way because box2D is weird
	--MASS SCHENANAGINS END
	self.x, self.y = Pbody:getPosition()--Where we are now
	
	if Pshape:getType() == "polygon" then--Used for Grav2D:draw
		self.points = self.body:getWorldPoints(self.shape:getPoints())
	else
		self.radius = Pshape:getRadius()
	end
	
	self.world = Pbody:getWorld()--stores world
	return self--A good thing in a small package

end

function Grav2D.toPolar(x,y)--converts from cartesian x,y coords to polar r,θ
		local r=math.sqr(math.pow(x,2)+math.pow(y,2))
		local θ=math.atan(y/x)
		return r,θ
end
	
function Grav2D.toCartesian(r,θ)--does the oppisite, used to calculate the force coords
		local x=r*math.cos(θ)
		local y=r*math.sin(θ)
		return x,y
	end

function Grav2D:draw(mode)--Used to draw the representation of the object
	self.x,self.y=self.body:getPosition()--getPos
	if not(self.radius==nil) then--Circle or Polygon?
		love.graphics.circle(mode,self.x,self.y,self.radius)
	elseif not(self.points==nil) then
		love.graphics.polygon(mode,self.body:getWorldPoints(self.shape:getPoints()))
	else
		error("body is neither a Polygon nor a Circle. Come again!")
	end
end

function Grav2D:applyGrav()-- The big one
	local meanFg = 0
	local meanAng = 0
	local meanDist = 0
	local dist = 0
	local xDist = 0
	local yDist = 0
	local ang = 0
	local fG = 0
	local fX = 0
	local fY = 0
	local bodyCount = self.world:getBodyCount()
	local bodyList = self.world:getBodyList()
	--var init end :P
	for i, v in ipairs(bodyList) do--for all bodies:
		xDist = v:getX()-self.body:getX()--get the distance between me and it (X)
		yDist = v:getY()-self.body:getY()-- (Y)
		if(not(xDist==0 and yDist==0)) then-- if not at my coordinates
			dist = math.sqrt(math.pow(xDist,2)+math.pow(yDist,2))--get the true distance
			ang = math.atan2(yDist,xDist)--get the angle between me and it
			Fg =  self.body:getMass()*v:getMass()*(1/dist)--the force that should be applied 
			
			self.body:applyForce(Grav2D.toCartesian(Fg,ang))--do the thing. converts 'force' and angle to x and y
		
			meanAng = meanAng+ang--self explanitory
			meanFg = meanFg+Fg--^
			meanDist = meanDist+dist--^^
		end
	end
	
	meanAng = meanAng/(bodyCount-1)--take the avrage not including itself
	meanFg = meanFg/(bodyCount-1)--same fam
	
	debugReturn =  {["meanAng"] = meanAng*(180/math.pi), ["meanFg"] = meanFg, ["meanDist"] = meanDist, ["bodyCount"] = bodyCount}--Used to do stuff :D
	
	return debugReturn--Fancy debug stuff.
end